# Function parse

Parse an expression. Returns a node tree which can be compiled and evaluated.


## Syntax

```js
math.parse(expr)
math.parse(expr, nodes)
math.parse([expr1, expr2, expr3, ...])
math.parse([expr1, expr2, expr3, ...], nodes)
```

### Parameters

Parameter | Type | Description
--------- | ---- | -----------
`expr` | String &#124; String[] &#124; Matrix | 
`nodes` | Object<String, Node> | 

### Returns

Type | Description
---- | -----------
Node &#124; Node[] | node


## Examples

```js
var node = math.parse('sqrt(3^2 + 4^2)');
node.compile(math).eval(); // 5

var scope = {a: 3, b: 4}
var node = math.parse('a * b'); // 12
var code = node.compile(math);
code.eval(scope); // 12
scope.a = 5;
code.eval(scope); // 20

var nodes = math.parse(['a = 3', 'b = 4', 'a * b']);
nodes[2].compile(math).eval(); // 12
```



